﻿/*
 * $Id: ActorPools.cs,v 1.1.1.1 2009/09/20 13:25:16 kenta Exp $
 *
 * Copyright 2009 Kenta Cho. Some rights reserved.
 */
using System;
using System.Collections.Generic;
using SilverArcade.SilverSprite.Input;
using Abagames.Util;

namespace Abagames.Hirarion
{
    /// <summary>
    /// Updating and drawing each actor pool.
    /// </summary>
    public class ActorPools
    {
        private static ShipPool ships;
        private static MissilePool missiles;
        private static ExplosionPool explosions;
        private static ParticlePool particles, darkParticles;
        private static BonusPool bonuses;
        private static Stage stage;

        public static void Initialize()
        {
            Field.Initialize();
            particles = new ParticlePool(30000);
            darkParticles = new ParticlePool(60000);
            ships = new ShipPool(50, particles, darkParticles);
            bonuses = new BonusPool(100, ships, darkParticles);
            missiles = new MissilePool(100, ships, particles, darkParticles, bonuses);
            explosions = new ExplosionPool(100, ships, missiles, particles, darkParticles);
            stage = new Stage(ships);
            ships.SetParams(missiles, bonuses, explosions);
            missiles.SetParams(explosions);
        }

        public static void Start(int randomSeed)
        {
            ships.Clear();
            bonuses.Clear();
            missiles.Clear();
            explosions.Clear();
            particles.Clear();
            darkParticles.Clear();
            stage.Initialize();
            ships.AddPlayer();
        }

        public static void Update()
        {
            Keyboard.Update();
            Pad.Update();
            stage.Update();
            ships.Update();
            explosions.Update();
            missiles.Update();
            bonuses.Update();
            particles.Update();
            darkParticles.Update();
        }

        public static void Draw()
        {
            Field.Draw();
            darkParticles.Draw();
            particles.Draw();
        }
    }
}
