﻿/*
 * $Id: Bonus.cs,v 1.1.1.1 2009/09/20 13:25:16 kenta Exp $
 *
 * Copyright 2009 Kenta Cho. Some rights reserved.
 */
using System;
using System.Collections.Generic;
#if SILVERLIGHT
using SilverArcade.SilverSprite;
#else
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
#endif
using Abagames.Util;

namespace Abagames.Hirarion
{
    /// <summary>
    /// Bonus items.
    /// </summary>
    public class BonusPool : ActorPool<Bonus>
    {
        private ShipPool ships;
        private ParticlePool particles;

        public BonusPool(int n, ShipPool ships, ParticlePool particles)
            : base(n)
        {
            this.ships = ships;
            this.particles = particles;
        }

        public void Add(Vector2 p, float angle, float speed)
        {
            Bonus a = new Bonus();
            a.Pos = p;
            a.Vel.X = (float)Math.Sin(angle) * speed;
            a.Vel.Y = (float)Math.Cos(angle) * speed;
            a.Ticks = 300;
            Add(ref a);
        }

        public override bool Update(ref Bonus a)
        {
            a.Pos += a.Vel;
            a.Vel *= 0.98f;
            a.Angle += 0.001f * a.Ticks;
            if (ships.IsPlayerAlive)
            {
                Vector2 pp = ships.PlayerPos;
                if (a.InhalingTicks > 0)
                {
                    a.InhalingTicks++;
                    float d = Vector2.Distance(a.Pos, pp);
                    if (d < 10)
                    {
                        Frame.AddMultiplier();
                        Frame.BonusSe.Play();
                        return false;
                    }
                    a.Vel.X += (pp.X - a.Pos.X) / d * 0.03f * a.InhalingTicks;
                    a.Vel.Y += (pp.Y - a.Pos.Y) / d * 0.03f * a.InhalingTicks;
                }
                else if (Vector2.Distance(a.Pos, pp) < 50)
                {
                    a.InhalingTicks = 1;
                }
            }
            float ang = a.Angle;
            for (int i = 0; i < 3; i++)
            {
                particles.Add(a.Pos, ang, 1, 15, 0, 180, 0);
                ang += (float)Math.PI * 2 / 3;
            }
            return (a.Ticks-- > 0);
        }
    }

    public struct Bonus
    {
        public Vector2 Pos;
        public Vector2 Vel;
        public int Ticks;
        public int InhalingTicks;
        public float Angle;
    }
}
