﻿/*
 * $Id: Explosion.cs,v 1.1.1.1 2009/09/20 13:25:16 kenta Exp $
 *
 * Copyright 2009 Kenta Cho. Some rights reserved.
 */
using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
#if SILVERLIGHT
using SilverArcade.SilverSprite;
#else
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
#endif
using Abagames.Util;

namespace Abagames.Hirarion
{
    /// <summary>
    /// Explosions of ships and missiles.
    /// </summary>
    public class ExplosionPool : ActorPool<Explosion>
    {
        private const float baseSize = 64;
        private Random random = new Random();
        private ShipPool ships;
        private MissilePool missiles;
        private ParticlePool particles, darkParticles;

        public ExplosionPool(int n, ShipPool ships, MissilePool missiles, ParticlePool particles, ParticlePool darkParticles)
            : base(n)
        {
            this.ships = ships;
            this.missiles = missiles;
            this.particles = particles;
            this.darkParticles = darkParticles;
        }

        public void Add(Vector2 p)
        {
            Explosion a = new Explosion();
            a.Pos = p;
            a.Scale = 1;
            Add(ref a);
            Frame.ExplosionSe.Play();
        }

        public override bool Update(ref Explosion a)
        {
            a.Ticks++;
            if (a.Ticks < 60)
                a.Scale = 1 + a.Ticks * 0.1f;
            else
                a.Scale = 1 + (120 - a.Ticks) * 0.1f;
            a.Scale /= 4;
            ships.CheckHit(a.Pos, baseSize / 3 * a.Scale, true);
            missiles.CheckHit(a.Pos, baseSize / 3 * a.Scale);
            for (int i = 0; i < 25; i++)
            {
                float ang = (float)(random.NextDouble() * Math.PI * 2);
                particles.Add(a.Pos, ang, a.Scale * 2, 15, 255, 255, 255);
                for (int j = 0; j < 5; j++)
                {
                    darkParticles.AddRandom(a.Pos, ang, a.Scale * 4, 10, 220, 220, 220);
                }
            }
            return (a.Ticks < 120);
        }
    }

    public struct Explosion
    {
        public Vector2 Pos;
        public float Scale;
        public int Ticks;
    }
}
