﻿/*
 * $Id: Frame.cs,v 1.1.1.1 2009/09/20 13:25:16 kenta Exp $
 *
 * Copyright 2009 Kenta Cho. Some rights reserved.
 */
using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using SilverArcade.SilverSprite.Audio;
using Abagames.Util;

namespace Abagames.Hirarion
{
    /// <summary>
    /// Managing a basic game cycle, audios and texts.
    /// </summary>
    public class Frame : Abagames.Util.Frame
    {
        public static SoundEffect ExplosionSe, MissileSe, BonusSe;
        private static Random random = new Random();
        private static SoundEffect bgm;
        private static SoundEffectInstance bgmInst;
        private static float bgmVolume;
        private static bool isBgmFadeOut;
        private static Canvas root;
        private static int score, multiplier;
        private static TextBlock scoreText, multiplierText, titleText, gameOverText, pushButtonText;
        private static int gameOverTicks, titleTicks;

        public static void Initialize(Canvas rt)
        {
            Initialize(Update, Draw);
            root = rt;
            ActorPools.Initialize();
            bgm = new SoundEffect(@"Content/Audio/ho1", rt);
            ExplosionSe = new SoundEffect(@"Content/Audio/explosion", rt);
            MissileSe = new SoundEffect(@"Content/Audio/missile", rt);
            BonusSe = new SoundEffect(@"Content/Audio/bonus", rt);
            scoreText = CreateTextBlock(20, 15, 15);
            multiplierText = CreateTextBlock(Screen.Width - 100 - 20, 15, 15);
            titleText = CreateTextBlock(Screen.Width - 100 - 20, Screen.Height / 2, 25);
            titleText.Visibility = Visibility.Collapsed;
            titleText.Text = "Hirarion";
            gameOverText = CreateTextBlock(Screen.Width / 2 - 50, Screen.Height / 2, 20);
            gameOverText.Visibility = Visibility.Collapsed;
            gameOverText.Text = "GAME OVER";
            pushButtonText = CreateTextBlock(Screen.Width - 100 - 20, Screen.Height / 2 + 50, 12);
            pushButtonText.Visibility = Visibility.Collapsed;
            pushButtonText.Text = "Push [Z] to start";
            gameOverTicks = 1;
        }

        private static TextBlock CreateTextBlock(int x, int y, int sz)
        {
            TextBlock tb = new TextBlock();
            tb.Foreground = new SolidColorBrush(Color.FromArgb(255, 255, 255, 255));
            tb.TextAlignment = TextAlignment.Right;
            tb.Width = 100;
            tb.Height = sz;
            tb.FontSize = sz;
            tb.FontFamily = new FontFamily("Tahoma");
            Canvas.SetLeft(tb, x);
            Canvas.SetTop(tb, y);
            root.Children.Add(tb);
            return tb;
        }

        public static void Start()
        {
            gameOverTicks = -1;
            gameOverText.Visibility = Visibility.Collapsed;
            titleText.Visibility = Visibility.Collapsed;
            pushButtonText.Visibility = Visibility.Collapsed;
            ActorPools.Start(random.Next());
            scoreText.Text = "0";
            multiplierText.Text = "x1";
            score = 0;
            multiplier = 1;
            if (bgmInst != null)
                bgmInst.Stop();
            bgmVolume = 1;
            isBgmFadeOut = false;
            //bgm = new SoundEffect(@"Content/Audio/ho1");
            bgmInst = bgm.Play(bgmVolume, true);
        }

        public static void StartGameOver()
        {
            FadeOutBgm();
            gameOverTicks = 300;
            gameOverText.Visibility = Visibility.Visible;
        }

        public static void Update()
        {
            ActorPools.Update();
            if (isBgmFadeOut)
            {
                bgmVolume -= 0.01f;
                if (bgmVolume <= 0)
                {
                    bgmInst.Stop();
                    isBgmFadeOut = false;
                }
                else
                {
                    bgmInst.Volume = bgmVolume;
                }
            }
            if (gameOverTicks >= 0)
            {
                if (gameOverTicks > 0)
                {
                    gameOverTicks--;
                    if (gameOverTicks <= 0)
                    {
                        gameOverText.Visibility = Visibility.Collapsed;
                        titleText.Visibility = Visibility.Visible;
                    }
                }
                else
                {
                    if (titleTicks % 120 == 0)
                    {
                        pushButtonText.Visibility = Visibility.Visible;
                    }
                    else if (titleTicks % 120 == 60)
                    {
                        pushButtonText.Visibility = Visibility.Collapsed;
                    }
                    titleTicks++;
                }
                if (Pad.ButtonAny)
                    Start();
            }
        }

        public static void AddScore(int s)
        {
            if (gameOverTicks >= 0)
                return;
            score += s * multiplier;
            scoreText.Text = score.ToString();
        }

        public static void AddMultiplier()
        {
            if (gameOverTicks >= 0)
                return;
            multiplier++;
            multiplierText.Text = "x" + multiplier;
        }

        public static void Draw()
        {
            Screen.Clear();
            ActorPools.Draw();
            Screen.Invalidate();
        }

        public static void FadeOutBgm()
        {
            isBgmFadeOut = true;
        }
    }
}
