﻿/*
 * $Id: Missile.cs,v 1.1.1.1 2009/09/20 13:25:16 kenta Exp $
 *
 * Copyright 2009 Kenta Cho. Some rights reserved.
 */
using System;
using System.Collections.Generic;
#if SILVERLIGHT
using SilverArcade.SilverSprite;
#else
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
#endif
using Abagames.Util;

namespace Abagames.Hirarion
{
    /// <summary>
    /// Enemies' missiles.
    /// </summary>
    public class MissilePool : ActorPool<Missile>
    {
        private Random random = new Random();
        private ShipPool ships;
        private ParticlePool particles, darkParticles;
        private BonusPool bonuses;
        private ExplosionPool explosions;

        public MissilePool(int n, ShipPool ships, ParticlePool particles, ParticlePool darkParticles, BonusPool bonuses)
            : base(n)
        {
            this.ships = ships;
            this.particles = particles;
            this.darkParticles = darkParticles;
            this.bonuses = bonuses;
        }

        public void SetParams(ExplosionPool explosions)
        {
            this.explosions = explosions;
        }

        public void Add(Vector2 p, float angle, MissileSpec spec)
        {
            Missile a = new Missile();
            a.Pos = p;
            a.Angle = angle;
            a.Spec = spec;
            a.Pos.X += (float)Math.Sin(a.Angle) * 2;
            a.Pos.Y += (float)Math.Cos(a.Angle) * 2;
            Add(ref a);
            Frame.MissileSe.Play();
        }

        public override bool Update(ref Missile a)
        {
            a.Pos.X += (float)Math.Sin(a.Angle) * a.Spec.Speed;
            a.Pos.Y += (float)Math.Cos(a.Angle) * a.Spec.Speed;
            a.Ticks++;
            if (a.Ticks % 7 == 0)
            {
                if (ships.IsPlayerAlive && Vector2.Distance(a.Pos, ships.PlayerPos) < 50)
                    bonuses.Add(a.Pos, (float)Math.Atan2(a.Pos.X - ships.PlayerPos.X, a.Pos.Y - ships.PlayerPos.Y), 3);
            }
            if ((a.Ticks % a.Spec.TurnDuration) < a.Spec.TurnInterval)
            {
                Vector2 ap = ships.PlayerPos;
                float d = Vector2.Distance(a.Pos, ap);
                d *= 0.5f;
                ap.X += (float)Math.Sin(ships.PlayerAngle) * d;
                ap.Y += (float)Math.Cos(ships.PlayerAngle) * d;
                float aa = (float)Math.Atan2(ap.X - a.Pos.X, ap.Y - a.Pos.Y);
                float ao = MathUtil.NormalizeAngle(aa - a.Angle);
                if (ao > a.Spec.AngleVel)
                    a.Angle += a.Spec.AngleVel;
                else if (ao < -a.Spec.AngleVel)
                    a.Angle -= a.Spec.AngleVel;
                else
                    a.Angle = aa;
            }
            if (ships.CheckHit(a.Pos, 2.5f, false))
            {
                explosions.Add(a.Pos);
                return false;
            }
            for (int i = 0; i < actorCount; i++)
            {
                float d = Vector2.Distance(a.Pos, Actors[i].Pos);
                if (d > 0.01f && d < 5)
                {
                    explosions.Add(a.Pos);
                    if (ships.IsPlayerAlive)
                        Frame.AddScore(1);
                    return false;
                }
            }
            for (int i = 0; i < 5; i++)
            {
                particles.Add(a.Pos, a.Angle, 1 + i, 60, 255, 255, 0);
                for (int j = 0; j < 3; j++)
                {
                    darkParticles.AddRandom(a.Pos, a.Angle, 20, 20, 180, 180, 0);
                }
            }
            return Field.ContainsOuter(a.Pos);
        }

        public void CheckHit(Vector2 p, float distance)
        {
            for (int i = 0; i < actorCount; i++)
            {
                if (Vector2.Distance(p, Actors[i].Pos) < distance)
                {
                    for (int j = 0; j < 10; j++)
                    {
                        bonuses.Add(Actors[i].Pos, (float)(random.NextDouble() * Math.PI * 2), 1 + (float)random.NextDouble() * 2);
                    }
                    explosions.Add(Actors[i].Pos);
                    if (ships.IsPlayerAlive)
                        Frame.AddScore(1);
                    Remove(i);
                    i--;
                }
            }
        }
    }

    public struct Missile
    {
        public Vector2 Pos;
        public float Angle;
        public int Ticks;
        public MissileSpec Spec;
    }

    public struct MissileSpec
    {
        public float Speed;
        public float AngleVel;
        public int TurnInterval, TurnDuration;
    }
}
