﻿/*
 * $Id: Particle.cs,v 1.1.1.1 2009/09/20 13:25:16 kenta Exp $
 *
 * Copyright 2009 Kenta Cho. Some rights reserved.
 */
using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using SilverArcade.SilverSprite;
using Abagames.Util;

namespace Abagames.Hirarion
{
    /// <summary>
    /// Particles.
    /// </summary>
    public class ParticlePool : ActorPool<Particle>
    {
        private Random random = new Random();

        public ParticlePool(int n)
            : base(n)
        {
        }

        public void Add(Vector2 p, float ang, float spd, int t, int r, int g, int b)
        {
            Particle a = new Particle();
            a.Pos = p;
            a.Vel.X = -(float)Math.Sin(ang) * spd;
            a.Vel.Y = -(float)Math.Cos(ang) * spd;
            a.Ticks = t;
            a.R = r;
            a.G = g;
            a.B = b;
            Add(ref a);
        }

        public void AddRandom(Vector2 p, float ang, float spd, int t, int r, int g, int b)
        {
            ang += (float)random.NextDouble() - 0.5f;
            spd *= (float)random.NextDouble();
            Add(p, ang, spd, t, r, g, b);
        }

        public override bool Update(ref Particle a)
        {
            a.Pos += a.Vel;
            a.Vel *= 0.99f;
            if (a.R > 0)
                a.R--;
            if (a.G > 0)
                a.G--;
            if (a.B > 0)
                a.B--;
            return (a.Ticks-- > 0);
        }

        public override void Draw(ref Particle a)
        {
            int x = (int)(a.Pos.X + Field.ScreenOffset.X);
            int y = (int)(a.Pos.Y + Field.ScreenOffset.Y);
            if (x < 0 || x >= Screen.Width || y < 0 || y >= Screen.Height)
                return;
            Screen.Bitmap.Pixels[x + y * Screen.Width] = (0xff << 24) | (a.R << 16) | (a.G << 8) | a.B;
        }
    }

    public struct Particle
    {
        public Vector2 Pos;
        public Vector2 Vel;
        public int R, G, B;
        public int Ticks;
    }
}
