﻿/*
 * $Id: Ship.cs,v 1.1.1.1 2009/09/20 13:25:16 kenta Exp $
 *
 * Copyright 2009 Kenta Cho. Some rights reserved.
 */
using System;
using System.Collections.Generic;
#if SILVERLIGHT
using SilverArcade.SilverSprite;
#else
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
#endif
using Abagames.Util;

namespace Abagames.Hirarion
{
    /// <summary>
    /// Player's and enemies' ships.
    /// </summary>
    public class ShipPool : ActorPool<Ship>
    {
        private Random random = new Random();
        private MissilePool missiles;
        private ParticlePool particles, darkParticles;
        private BonusPool bonuses;
        private ExplosionPool explosions;
        private bool isPlayerAlive;

        public ShipPool(int n, ParticlePool particles, ParticlePool darkParticles)
            : base(n)
        {
            this.particles = particles;
            this.darkParticles = darkParticles;
        }

        public void SetParams(MissilePool missiles, BonusPool bonuses, ExplosionPool explosions)
        {
            this.missiles = missiles;
            this.bonuses = bonuses;
            this.explosions = explosions;
        }

        public void AddPlayer()
        {
            Ship a = new Ship();
            a.Pos = Vector2.Zero;
            a.Angle = (float)Math.PI;
            a.Speed = 2;
            a.IsPlayer = true;
            isPlayerAlive = true;
            Add(ref a);
        }

        public void Add(Vector2 p, float angle, float speed, float angleVel, MissileSpec missileSpec)
        {
            Ship a = new Ship();
            a.Pos = p;
            a.Angle = angle;
            a.Speed = speed;
            a.AngleVel = angleVel;
            a.MissileSpec = missileSpec;
            Add(ref a);
        }

        public override bool Update(ref Ship a)
        {
            a.Pos.X += (float)Math.Sin(a.Angle) * a.Speed;
            a.Pos.Y += (float)Math.Cos(a.Angle) * a.Speed;
            a.Ticks++;
            if (a.IsPlayer)
            {
                for (int i = 0; i < 7; i++)
                {
                    particles.Add(a.Pos, a.Angle - 0.01f * (i - 3), 2, 60, 0, 255, 255);
                    for (int j = 0; j < 5; j++)
                    {
                        darkParticles.AddRandom(a.Pos, a.Angle - 0.02f * (i - 4), 10, 30, 0, 200, 200);
                    }
                }
            }
            else
            {
                for (int i = 0; i < 5; i++)
                {
                    particles.Add(a.Pos, a.Angle - 0.02f * (i - 2), 2, 60, 255, 0, 255);
                    for (int j = 0; j < 3; j++)
                    {
                        darkParticles.AddRandom(a.Pos, a.Angle - 0.02f * (i - 4), 10, 30, 180, 0, 180);
                    }
                }
            }
            if (a.IsPlayer)
            {
                UpdatePlayer(ref a);
                return true;
            }
            if (Field.Contains(a.Pos))
            {
                float pa = (float)Math.Atan2(PlayerPos.X - a.Pos.X, PlayerPos.Y - a.Pos.Y);
                TurnTo(ref a, pa, a.AngleVel);
                if (a.Ticks % 80 == 0)
                {
                    Vector2 ap = GetAimed(a.Pos, a.Angle, true, 150);
                    if (ap.X < 99999)
                    {
                        float ma;
                        if (a.Speed >= a.MissileSpec.Speed)
                            ma = pa;
                        else
                            ma = a.Angle;
                        missiles.Add(a.Pos, ma, a.MissileSpec);
                    }
                }
            }
            return Field.ContainsOuter(a.Pos);
        }

        private void UpdatePlayer(ref Ship a)
        {
            if (Field.Contains(a.Pos))
            {
                float px = Pad.ThumbStickLeft.X + Pad.ThumbStickRight.X;
                if (px > 1)
                    px = 1;
                else if (px < -1)
                    px = -1;
                a.Angle -= px * 0.05f;
            }
            else
            {
                TurnTo(ref a, (float)Math.Atan2(-a.Pos.X, -a.Pos.Y), 0.1f);
            }
            Field.ScreenOffset.X = Screen.Width / 2 - a.Pos.X * 0.4f;
            Field.ScreenOffset.Y = Screen.Height / 2 - a.Pos.Y * 0.4f;
        }

        private void TurnTo(ref Ship a, float ta, float av)
        {
            float ao = MathUtil.NormalizeAngle(ta - a.Angle);
            if (ao > av)
                a.Angle += av;
            else if (ao < -av)
                a.Angle -= av;
            else
                a.Angle = ta;
        }

        public Vector2 GetAimed(Vector2 p, float angle, bool isAimingPlayer, float minDist)
        {
            float md = 300;
            Vector2 ap = new Vector2(9999999, 0);
            ForEach(delegate(ref Ship a)
            {
                float ao = MathUtil.NormalizeAngle(angle - (float)Math.Atan2(a.Pos.X - p.X, a.Pos.Y - p.Y));
                if (Math.Abs(ao) < (float)Math.PI / 3)
                {
                    float d = Vector2.Distance(p, a.Pos);
                    if (d > minDist && d < md)
                    {
                        if (isAimingPlayer)
                        {
                            if (a.IsPlayer)
                            {
                                md = d;
                                ap = a.Pos;
                            }
                            else
                            {
                                md = 0;
                                ap = new Vector2(9999999, 0);
                            }
                        }
                        else
                        {
                            md = d;
                            ap = a.Pos;
                        }
                    }
                }
            });
            return ap;
        }

        public bool CheckHit(Vector2 p, float distance, bool isRemoving)
        {
            bool isHit = false;
            for (int i = 0; i < actorCount; i++)
            {
                if (Vector2.Distance(p, Actors[i].Pos) < distance)
                {
                    if (!isRemoving)
                        return true;
                    if (isPlayerAlive && i == 0)
                    {
                        isPlayerAlive = false;
                        Frame.StartGameOver();
                    }
                    for (int j = 0; j < 20; j++)
                    {
                        bonuses.Add(Actors[i].Pos, (float)(random.NextDouble() * Math.PI * 2), 1 + (float)random.NextDouble() * 2);
                    }
                    explosions.Add(Actors[i].Pos);
                    Frame.AddScore(10);
                    Remove(i);
                    i--;
                    isHit = true;
                }
            }
            return isHit;
        }

        public override void Draw(ref Ship a)
        {
        }

        public Vector2 PlayerPos
        {
            get
            {
                return Actors[0].Pos;
            }
        }

        public float PlayerAngle
        {
            get
            {
                return Actors[0].Angle;
            }
        }

        public bool IsPlayerAlive
        {
            get
            {
                return isPlayerAlive;
            }
        }
    }

    public struct Ship
    {
        public Vector2 Pos;
        public float Angle;
        public float AngleVel;
        public float Speed;
        public MissileSpec MissileSpec;
        public int Ticks;
        public bool IsPlayer;
    }
}
