/*
 * $Id: Stage.cs,v 1.1.1.1 2009/09/20 13:25:16 kenta Exp $
 *
 * Copyright 2009 Kenta Cho. Some rights reserved.
 */
using System;
#if SILVERLIGHT
using SilverArcade.SilverSprite;
#else
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
#endif
using Abagames.Util;

namespace Abagames.Hirarion
{
    /// <summary>
    /// Handling enemy ship appearances.
    /// </summary>
    public class Stage
    {
        private Random random = new Random();
        private ShipPool ships;
        private int appTicks;
        private float shipSpeed, shipAngleVel;
        private MissileSpec missileSpec;
        private float appInterval;

        public Stage(ShipPool ships)
        {
            this.ships = ships;
        }

        public void Initialize()
        {
            appTicks = 0;
            SetSpecs();
        }

        public void Update()
        {
            if (!ships.IsPlayerAlive)
                return;
            appTicks--;
            if (appTicks <= 0)
            {
                Vector2 p, o;
                if (random.Next(2) == 0)
                {
                    p.X = (float)random.NextDouble() * Field.Size.X;
                    p.Y = Field.SizeOuter.Y;
                    o.X = 1;
                    o.Y = 0;
                }
                else
                {
                    p.X = Field.SizeOuter.X;
                    p.Y = (float)random.NextDouble() * Field.Size.Y;
                    o.X = 0;
                    o.Y = 1;
                }
                p.X *= (random.Next(2) * 2 - 1) * 0.99f;
                p.Y *= (random.Next(2) * 2 - 1) * 0.99f;
                float a = (float)Math.Atan2(-p.X, -p.Y);
                ships.Add(p, a, shipSpeed, shipAngleVel, missileSpec);
                appTicks = (int)appInterval;
            }
            if (appInterval > 10)
                appInterval -= 0.02f;
            missileSpec.AngleVel += 0.00005f;
            missileSpec.Speed += 0.00001f;
        }

        private void SetSpecs()
        {
            shipSpeed = 1;
            shipAngleVel = 0.01f;
            missileSpec.Speed = 2.1f;
            missileSpec.AngleVel = 0;
            missileSpec.TurnInterval = 5;
            missileSpec.TurnDuration = 30;
            appInterval = 120;
        }
    }
}
