/*
 * $Id: Actor.cs,v 1.1.1.1 2009/09/20 13:25:16 kenta Exp $
 *
 * Copyright 2009 Kenta Cho. Some rights reserved.
 */
using System;
using System.Collections;
using System.Collections.Generic;

namespace Abagames.Util
{
    /// <summary>
    /// Actor instances pooled in an ActorPool.
    /// </summary>
    public class ActorPool<T>
        where T : struct
    {
        public T[] Actors;
        protected int actorCount;
#if !SILVERLIGHT
        protected Shape shape;
#endif

        public ActorPool(int n)
        {
            Actors = new T[n];
        }

        public virtual void Clear()
        {
            actorCount = 0;
        }

        public int Add(ref T a)
        {
            if (actorCount >= Actors.Length)
                return -1;
            Actors[actorCount] = a;
            actorCount++;
            return actorCount - 1;
        }

        public virtual void Remove(int index)
        {
            actorCount--;
            Actors[index] = Actors[actorCount];
        }

        public virtual void Update()
        {
            for (int i = 0; i < actorCount;)
            {
                if (Update(ref Actors[i]))
                    i++;
                else
                    Remove(i);
            }
        }

        public virtual void Draw()
        {
#if SILVERLIGHT
            ForEach(delegate(ref T a)
            {
                Draw(ref a);
            });
#else
            Draw(shape);
#endif
        }

#if !SILVERLIGHT
        public virtual void Draw(Shape shape)
        {
            if (actorCount <= 0)
                return;
            int si = 0, nsi;
            for (; ; )
            {
                nsi = si + Shape.MaxInstanceCount;
                if (nsi > actorCount)
                    nsi = actorCount;
#if XBOX360
                Shape.SetToWritableVertexBuffer<T>(Actors, si, nsi - si);
#else
                Shape.BeginAddInstance();
                for (int i = si; i < nsi; i++)
                {
                    Draw(ref Actors[i]);
                }
#endif
                shape.Draw();
                if (nsi >= actorCount)
                    break;
                si = nsi;
            }
        }
#endif

        public virtual bool Update(ref T a)
        {
            return true;
        }

        public virtual void Draw(ref T a)
        {
        }

        public T this[int idx]
        {
            get
            {
                return Actors[idx];
            }
        }

        public int Count
        {
            get
            {
                return actorCount;
            }
        }

        public bool IsAbleToAdd
        {
            get
            {
                return (actorCount < Actors.Length);
            }
        }

        public int RemainingActorCount
        {
            get
            {
                return Actors.Length - actorCount;
            }
        }

        public delegate void MyAction<U>(ref U obj);

        public void ForEach(MyAction<T> func)
        {
            for (int i = 0; i < actorCount; i++)
            {
                func(ref Actors[i]);
            }
        }
    }
}
