/*
 * $Id: Pad.cs,v 1.1.1.1 2009/09/20 13:25:16 kenta Exp $
 *
 * Copyright 2009 Kenta Cho. Some rights reserved.
 */
using System;
using System.Collections.Generic;
#if SILVERLIGHT
using SilverArcade.SilverSprite;
using SilverArcade.SilverSprite.Input;
#else
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
#endif

namespace Abagames.Util
{
    /// <summary>
    /// Inputs from a game pad and a keyboard.
    /// </summary>
    public class Pad
    {
        public static float SensitivityThreshold = 0.2f;
        private static GamePadState gamePadState;
        private static KeyboardState keyboardState;
        private static PlayerIndex playerIndex = PlayerIndex.One;

        public static void Update()
        {
            keyboardState = Keyboard.GetState();
            gamePadState = GamePad.GetState(playerIndex);
        }

        public static int CheckGameStartPressed()
        {
            if (keyboardState.IsKeyDown(Keys.C) || keyboardState.IsKeyDown(Keys.OemPeriod) ||
                    keyboardState.IsKeyDown(Keys.V) || keyboardState.IsKeyDown(Keys.OemQuestion) ||
                    keyboardState.IsKeyDown(Keys.Z) || keyboardState.IsKeyDown(Keys.M) ||
                    keyboardState.IsKeyDown(Keys.X) || keyboardState.IsKeyDown(Keys.OemComma) ||
                    keyboardState.IsKeyDown(Keys.F1) || keyboardState.IsKeyDown(Keys.P))
                return 99;
            for (int i = 0; i < 4; i++)
            {
                GamePadState gamePadState = GamePad.GetState((PlayerIndex)i);
                if (!gamePadState.IsConnected)
                    continue;
                if (gamePadState.Buttons.A == ButtonState.Pressed || gamePadState.Buttons.B == ButtonState.Pressed ||
                    gamePadState.Buttons.X == ButtonState.Pressed || gamePadState.Buttons.Y == ButtonState.Pressed ||
                    gamePadState.Buttons.LeftShoulder == ButtonState.Pressed ||
                    gamePadState.Buttons.RightShoulder == ButtonState.Pressed ||
                    gamePadState.Buttons.Start == ButtonState.Pressed)
                    return i;
            }
            return -1;
        }

        public static void SetGamePadPlayerIndex(int i)
        {
            playerIndex = (PlayerIndex)i;
        }

        public static Vector2 ThumbStickLeft
        {
            get
            {
                Vector2 direction = new Vector2(0, 0);
                if (gamePadState.IsConnected)
                    direction = gamePadState.ThumbSticks.Left;
                if (keyboardState.IsKeyDown(Keys.W))
                    direction.Y = 1;
                if (keyboardState.IsKeyDown(Keys.A))
                    direction.X = -1;
                if (keyboardState.IsKeyDown(Keys.S))
                    direction.Y = -1;
                if (keyboardState.IsKeyDown(Keys.D))
                    direction.X = 1;
                if (Math.Abs(direction.X) < SensitivityThreshold)
                    direction.X = 0;
                if (Math.Abs(direction.Y) < SensitivityThreshold)
                    direction.Y = 0;
                return direction;
            }
        }

        public static Vector2 ThumbStickRight
        {
            get
            {
                Vector2 direction = new Vector2(0, 0);
                if (gamePadState.IsConnected)
                    direction = gamePadState.ThumbSticks.Right;
                if (keyboardState.IsKeyDown(Keys.Up) || keyboardState.IsKeyDown(Keys.NumPad8))
                    direction.Y = 1;
                if (keyboardState.IsKeyDown(Keys.Left) || keyboardState.IsKeyDown(Keys.NumPad4))
                    direction.X = -1;
                if (keyboardState.IsKeyDown(Keys.Down) || keyboardState.IsKeyDown(Keys.NumPad2))
                    direction.Y = -1;
                if (keyboardState.IsKeyDown(Keys.Right) || keyboardState.IsKeyDown(Keys.NumPad6))
                    direction.X = 1;
                if (Math.Abs(direction.X) < SensitivityThreshold)
                    direction.X = 0;
                if (Math.Abs(direction.Y) < SensitivityThreshold)
                    direction.Y = 0;
                return direction;
            }
        }

        public static bool ButtonAny
        {
            get
            {
                return ((gamePadState.IsConnected &&
                    (gamePadState.Buttons.A == ButtonState.Pressed || gamePadState.Buttons.B == ButtonState.Pressed ||
                     gamePadState.Buttons.X == ButtonState.Pressed || gamePadState.Buttons.Y == ButtonState.Pressed ||
                     gamePadState.Buttons.LeftShoulder == ButtonState.Pressed ||
                     gamePadState.Buttons.RightShoulder == ButtonState.Pressed)) ||
                    keyboardState.IsKeyDown(Keys.C) || keyboardState.IsKeyDown(Keys.OemPeriod) ||
                    keyboardState.IsKeyDown(Keys.V) || keyboardState.IsKeyDown(Keys.OemQuestion) ||
                    keyboardState.IsKeyDown(Keys.Z) || keyboardState.IsKeyDown(Keys.M) ||
                    keyboardState.IsKeyDown(Keys.X) || keyboardState.IsKeyDown(Keys.OemComma));
            }
        }

        public static bool ButtonA
        {
            get
            {
                return ((gamePadState.IsConnected &&
                    (gamePadState.Buttons.A == ButtonState.Pressed || gamePadState.Buttons.X == ButtonState.Pressed)) ||
                    keyboardState.IsKeyDown(Keys.Z) || keyboardState.IsKeyDown(Keys.M));
            }
        }

        public static bool ButtonB
        {
            get
            {
                return ((gamePadState.IsConnected &&
                    (gamePadState.Buttons.B == ButtonState.Pressed || gamePadState.Buttons.Y == ButtonState.Pressed)) ||
                    keyboardState.IsKeyDown(Keys.X) || keyboardState.IsKeyDown(Keys.OemComma));
            }
        }


        public static bool ButtonL
        {
            get
            {
                return ((gamePadState.IsConnected && gamePadState.Buttons.LeftShoulder == ButtonState.Pressed) ||
                    keyboardState.IsKeyDown(Keys.C) || keyboardState.IsKeyDown(Keys.OemPeriod));
            }
        }

        public static bool ButtonR
        {
            get
            {
                return ((gamePadState.IsConnected && gamePadState.Buttons.RightShoulder == ButtonState.Pressed) ||
                    keyboardState.IsKeyDown(Keys.V) || keyboardState.IsKeyDown(Keys.OemQuestion));
            }
        }

        public static float LeftTrigger
        {
            get
            {
                float r = 0;
                if (gamePadState.IsConnected)
                {
                    r = gamePadState.Triggers.Left;
                    if (r < SensitivityThreshold)
                        r = 0;
                }
                if (keyboardState.IsKeyDown(Keys.Q))
                    r = 1;
                return r;
            }
        }

        public static float RightTrigger
        {
            get
            {
                float r = 0;
                if (gamePadState.IsConnected)
                {
                    r = gamePadState.Triggers.Right;
                    if (r < SensitivityThreshold)
                        r = 0;
                }
                if (keyboardState.IsKeyDown(Keys.E))
                    r = 1;
                return r;
            }
        }

        public static bool ButtonStart
        {
            get
            {
                return ((gamePadState.IsConnected && gamePadState.Buttons.Start == ButtonState.Pressed) ||
                    keyboardState.IsKeyDown(Keys.F1) || keyboardState.IsKeyDown(Keys.P));
            }
        }

        public static bool ButtonBack
        {
            get
            {
                return ((gamePadState.IsConnected && gamePadState.Buttons.Back == ButtonState.Pressed) ||
                    keyboardState.IsKeyDown(Keys.Escape));
            }
        }

        public static bool ButtonFullScreen
        {
            get
            {
                return (keyboardState.IsKeyDown(Keys.Enter) && (keyboardState.IsKeyDown(Keys.LeftAlt) || keyboardState.IsKeyDown(Keys.RightAlt)));
            }
        }
    }
}
